package com.game.business.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.game.business.domain.*;
import com.game.business.mapper.AppUserCountMapper;
import com.game.business.service.*;
import com.game.common.constant.finance.FinTranType1;
import com.game.common.constant.finance.FinTranType3;
import com.game.common.entity.KeyValue;
import com.game.common.utils.DateUtils;
import com.game.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.Date;

@Service
public class AppUserCountServiceImpl extends ServiceImpl<AppUserCountMapper, AppUserCount> implements IAppUserCountService {

    @Autowired
    private AppUserCountMapper appUserCountMapper;

    @Autowired
    private IAppUserService appUserService;

    @Autowired
    private IAppUsersChargeService appUsersChargeService;

    @Autowired
    private IAppUsersCashrecordService appUsersCashrecordService;

    @Autowired
    private IFinTranRecordService finTranRecordService;

    @Autowired
    private IAppUserLiveDividedRecordService appUserLiveDividedRecordService;

    @Autowired
    private IAppUserGameRecordCountService appUserGameRecordCountService;

    @Autowired
    private IAppGameBettingService appGameBettingService;

    @Override
    public List<AppUserCount> getAppUserCount(long userId,String beginTime,String endTIme) {
        LambdaQueryWrapper<AppUserCount> queryWrapper = new LambdaQueryWrapper<AppUserCount>();
        queryWrapper.between(AppUserCount::getStrDate,beginTime,endTIme);
        queryWrapper.eq(AppUserCount::getAgentUserId,userId);
        return appUserCountMapper.selectList(queryWrapper);
    }

    @Override
//    @Async
    @Transactional
    public void createAppUserCount(AppUserCount appUserCount) {
        //查询当前用户当日是否已经生成记录
        LambdaQueryWrapper<AppUserCount> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppUserCount::getStrDate,appUserCount.getStrDate());
        queryWrapper.eq(AppUserCount::getUserId,appUserCount.getUserId());
        AppUserCount userCount = appUserCountMapper.selectOne(queryWrapper);
        int res = 0;
        if(userCount==null) {
            appUserCount.setCreateTime(new Date());
            appUserCount.setUpdateTime(new Date());
            res = appUserCountMapper.insert(appUserCount);
        }else{
            appUserCount.setId(userCount.getId());
            /*if(null == appUserCount.getUpdateTime()) {
                appUserCount.setUpdateTime(new Date());
            }*/
            if(!appUserCount.isUpdate()) {
                //非全量更新
                if (appUserCount.getRechargeAmount() != null) {
                    appUserCount.setRechargeAmount(userCount.getRechargeAmount() + appUserCount.getRechargeAmount());
                }
                if (appUserCount.getRechargeCommission() != null) {
                    appUserCount.setRechargeCommission(userCount.getRechargeCommission() + appUserCount.getRechargeCommission());
                }
                if (appUserCount.getRechargeGive() != null) {
                    appUserCount.setRechargeGive(userCount.getRechargeGive() + appUserCount.getRechargeGive());
                }
                if (appUserCount.getWithdrawalCommission() != null) {
                    appUserCount.setWithdrawalAmount(userCount.getWithdrawalAmount() + appUserCount.getWithdrawalAmount());
                }
                if (appUserCount.getWithdrawalCommission() != null) {
                    appUserCount.setWithdrawalCommission(userCount.getWithdrawalCommission() + appUserCount.getWithdrawalCommission());
                }
                if (appUserCount.getLiveUseAmount() != null) {
                    appUserCount.setLiveUseAmount(userCount.getLiveUseAmount() + appUserCount.getLiveUseAmount());
                }
                if (appUserCount.getLiveCommission() != null) {
                    appUserCount.setLiveCommission(userCount.getLiveCommission() + appUserCount.getLiveCommission());
                }
                if (appUserCount.getGameWinAmount() != null) {
                    appUserCount.setGameWinAmount(userCount.getGameWinAmount() + appUserCount.getGameWinAmount());
                }
                if (appUserCount.getGameLoseAmount() != null) {
                    appUserCount.setGameLoseAmount(userCount.getGameLoseAmount() + appUserCount.getGameLoseAmount());
                }
                if (appUserCount.getGameCommission() != null) {
                    appUserCount.setGameCommission(userCount.getGameCommission() + appUserCount.getGameCommission());
                }
            }
            res = appUserCountMapper.updateById(appUserCount);
        }
    }

    @Override
    public void staticsUserCount(Long gameId,String dateTime,String gameDate) {
        Date curDate = DateUtils.getNowDate();
        if (StringUtils.isNotBlank(dateTime)) {
            curDate = DateUtil.parse(dateTime, "yyyy-MM-dd");
        } else {
            dateTime = DateUtils.parseDateToStr("yyyy-MM-dd", curDate);
        }

        /*Date lastDate = this.selectLastUpdateDate();
        String lastDateStr = DateUtil.format(lastDate,"yyyy-MM-dd HH:mm:ss");
        if(!dateTime.equals(DateUtil.format(lastDate,"yyyy-MM-dd"))){
            lastDateStr = DateUtil.format(curDate, "yyyy-MM-dd").concat(" 00:00:00");
        }
        String endTimeStr = DateUtil.format(curDate,"yyyy-MM-dd HH:mm:ss");*/
        AppUser queryUser = new AppUser();
        queryUser.setAgentFlag(1);
        List<AppUser> list = appUserService.selectAppUserList(queryUser);//查询所有代理用户
        List<AppUserGameRecordCount> gameRecordCountList = appUserGameRecordCountService.selectByDate(null, null, null,gameId,gameDate);
        List<AppGameBetting> gameBettingList = appGameBettingService.selectListByDate(null, null, null,gameId,gameDate);
        if (null != list && list.size() > 0) {
            String finalDateTime = dateTime;
            Date finalCurDate = curDate;
            list.forEach(appUser -> {
                AppUserCount appUserCount = new AppUserCount();
                appUserCount.setUserId(appUser.getUserid());
                appUserCount.setStrDate(finalDateTime);
                appUserCount.setAgentUserId(appUser.getPid());
                appUserCount.setUpdateTime(finalCurDate);
                //游戏佣金
                appUserCount.setGameCommission(gameRecordCountList.stream().filter(e -> e.getUserId().equals(appUser.getUserid())).mapToDouble(e -> e.getCommission()).sum());
                //游戏输
                appUserCount.setGameLoseAmount(gameBettingList.stream().filter(
                        e -> e.getUserId().equals(appUser.getUserid()) && e.getIsWinning().intValue() == 2
                ).mapToDouble(e ->
                        e.getBettingAmount()
                ).sum());
                //游戏赢
                appUserCount.setGameWinAmount(gameBettingList.stream().filter(
                        e -> e.getUserId().equals(appUser.getUserid()) && e.getIsWinning().intValue() == 1
                ).mapToDouble(e ->
                        e.getBettingAmount() * e.getBettingMultiple()
                ).sum());
                this.createAppUserCount(appUserCount);
            });
        }
    }

    @Override
    public Date selectLastUpdateDate() {
        LambdaQueryWrapper<AppUserCount> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.orderByDesc(AppUserCount::getUpdateTime);
        queryWrapper.last("limit 1");
        AppUserCount appUserCount = appUserCountMapper.selectOne(queryWrapper);
        if(null == appUserCount || null == appUserCount.getUpdateTime()) {
            return DateUtil.beginOfDay(DateUtils.getNowDate());
        }
        return DateUtil.offsetMillisecond(appUserCount.getUpdateTime(), 1);  //往前加1毫秒用于增量更新
    }

    /**
     * 查询最后更新时间
     * */
    @Override
    public Date selectLastDate() {
       LambdaQueryWrapper<AppUserCount> queryWrapper = new LambdaQueryWrapper<>();
       queryWrapper.orderByDesc(AppUserCount::getUpdateTime);
       queryWrapper.last("limit 1");
       AppUserCount appUserCount = appUserCountMapper.selectOne(queryWrapper);
       if(null == appUserCount || null == appUserCount.getCountTime()) {
           return DateUtil.beginOfDay(DateUtils.getNowDate());
       }
       return DateUtil.offsetMillisecond(appUserCount.getCountTime(), 1);  //往前加1毫秒用于增量更新
    }

    /**
     * 金额为负数则转为正数
     * */
    private double dealMoney(double money){
        return money<0?money*-1:money;
    }

}
